@@ -36,12 +36,8 @@
 
 #include <regex.h>
 
-#include "leakdetect.h"
 #include "gui.h"
 #include "memprof.h"
-#include "process.h"
-#include "profile.h"
-#include "server.h"
 #include "treeviewutils.h"
 #include <string.h>
 #include <stdlib.h>
@@ -49,34 +45,6 @@
 #include <glib/gprintf.h>
 #include "elfparser.h"
 
-struct _ProcessWindow {
-	MPProcess *process;
-	Profile *profile;
-	GSList *leaks;
-
-	GtkWidget *main_window;
-	GtkWidget *main_notebook;
-	GtkWidget *n_allocations_label;
-	GtkWidget *profile_status_label;
-	GtkWidget *bytes_per_label;
-	GtkWidget *total_bytes_label;
-
-	GtkWidget *profile_func_tree_view;
-	GtkWidget *profile_caller_tree_view;
-	GtkWidget *profile_descendants_tree_view;
-
-	GtkWidget *leak_block_tree_view;
-	GtkWidget *leak_stack_tree_view;
-
-	GtkWidget *usage_max_label;
-	GtkWidget *usage_area;
-
-	guint usage_max;
-	guint usage_high;
-	guint usage_leaked;
-
-	guint status_update_timeout;
-};
 
 enum {
 	PROFILE_FUNC_NAME,
@@ -253,7 +221,8 @@ update_status (gpointer data)
 	pwin->usage_high = MAX (pwin->process->bytes_used, pwin->usage_high);
 
 	gtk_widget_queue_draw (pwin->usage_area);
-	
+	dw_update(pwin);
+
 	return TRUE;
 }
 
@@ -494,12 +463,12 @@ profile_selection_changed (GtkTreeSelection *selection, ProcessWindow *pwin)
 
 		if (caller->node) {
 			if (caller->node->symbol)
-				name = caller->node->symbol;
+				name = elf_demangle(caller->node->symbol);
 			else
-				name = "???";
+				name = g_strdup("???");
 		}
 		else
-			name = "<spontaneous>";
+			name = g_strdup("<spontaneous>");
 			
 		gtk_list_store_append (list_store, &iter);
 			
@@ -510,6 +479,8 @@ profile_selection_changed (GtkTreeSelection *selection, ProcessWindow *pwin)
 
 		set_sample (list_model, &iter, PROFILE_CALLER_SELF, caller->self, n_samples);
 		set_sample (list_model, &iter, PROFILE_CALLER_TOTAL, caller->total, n_samples);
+
+		g_free(name);
 	}
 	profile_caller_list_free (caller_list);
 
@@ -571,9 +542,9 @@ profile_fill (ProcessWindow *pwin)
 		g_assert (func);
 
 		if (func->node->symbol)
-			name = func->node->symbol;
+			name = elf_demangle(func->node->symbol);
 		else
-			name = "???";
+			name = g_strdup("???");
 		
 		gtk_list_store_set (store, &iter,
 				    PROFILE_FUNC_NAME, name,
@@ -582,6 +553,8 @@ profile_fill (ProcessWindow *pwin)
 		
 		set_sample (model, &iter, PROFILE_FUNC_SELF, func->self, n_samples);
 		set_sample (model, &iter, PROFILE_FUNC_TOTAL, func->total, n_samples);
+
+		g_free(name);
 	}
 	
 	tree_view_set_sort_ids (GTK_TREE_VIEW (pwin->profile_func_tree_view));
@@ -649,7 +622,7 @@ leak_block_selection_changed (GtkTreeSelection *selection,
 		if (!process_find_line (pwin->process, stack->address,
 					&filename, &functionname, &line)) {
 			/* 0x3f == '?' -- suppress trigraph warnings */
-			functionname = g_strdup ("(\x3f\x3f\x3f)");
+			functionname = "(\x3f\x3f\x3f)";
 			filename = "(\x3f\x3f\x3f)";
 			line = 0;
 		}
@@ -682,6 +655,7 @@ leaks_fill (ProcessWindow *pwin)
 	tmp_list = pwin->leaks;
 	while (tmp_list) {
 		const char *filename;
+		gboolean free_function = FALSE;
 		char *functionname = NULL;
 		GtkTreeIter iter;
 
@@ -700,7 +674,6 @@ leaks_fill (ProcessWindow *pwin)
 
 				for (tmp_list = skip_funcs; tmp_list != NULL; tmp_list = tmp_list->next) {
 					if (!strcmp (functionname, tmp_list->data)) {
-						free (functionname);
 						functionname = NULL;
 						break;
 					}
@@ -715,7 +688,6 @@ leaks_fill (ProcessWindow *pwin)
 					regcomp (&regex, tmp_list->data, 0);
 
 					if (!regexec (&regex, functionname, 0, NULL, 0)) {
-						free (functionname);
 						functionname = NULL;
 						regfree (&regex);
 						break;
@@ -728,8 +700,10 @@ leaks_fill (ProcessWindow *pwin)
 			if (functionname)
 				break;
 		}
-		if (!functionname)
+		if (!functionname) {
+			free_function = TRUE;
 			functionname = g_strdup ("(\x3f\x3f\x3f)");
+		}
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
@@ -739,7 +713,8 @@ leaks_fill (ProcessWindow *pwin)
 				    LEAK_BLOCK_BLOCK, block,
 				    -1);
 
-		g_free (functionname);
+		if (free_function)
+			g_free (functionname);
 		
 		tmp_list = tmp_list->next;
 	}
@@ -789,13 +764,11 @@ leak_stack_run_command (ProcessWindow *pwin, Block *block, int frame)
 			p++;
 		}
 		
-		free (functionname);
-
 		cmdline = g_strdup_printf ("/bin/sh -c %s", command->str);
 
 		if (!g_spawn_command_line_async (cmdline, &err)) {
 			show_error (pwin->main_window,
-				    ERROR_MODAL, _("Executation of \"%s\" failed: %s"),
+				    ERROR_MODAL, _("Execution of \"%s\" failed: %s"),
 				    command->str, err->message);
 			
 			g_error_free (err);
@@ -971,6 +944,7 @@ process_window_reset (ProcessWindow *pwin)
 	gtk_window_set_title (GTK_WINDOW (pwin->main_window), "MemProf");
 
 	gtk_widget_queue_draw (pwin->usage_area);
+	dw_update(pwin);
 }
 
 static void
@@ -1049,17 +1023,13 @@ run_cb (GtkWidget *widget)
 {
        GladeXML *xml;
        GtkWidget *run_dialog;
-       GtkWidget *entry;
+       GtkWidget *filechooser;
 
        ProcessWindow *pwin = pwin_from_widget (widget);
        
        xml = glade_xml_new (glade_file, "RunDialog", NULL);
        run_dialog = get_widget (xml, "RunDialog");
-       entry = get_widget (xml, "RunDialog-entry");
-
-       g_signal_connect_swapped (entry, "activate",
-				 G_CALLBACK (gtk_widget_activate),
-				 get_widget (xml, "RunDialog-run"));
+       filechooser = get_widget (xml, "RunDialog-chooser");
 
        g_object_unref (G_OBJECT (xml));
 
@@ -1071,7 +1041,10 @@ run_cb (GtkWidget *widget)
 		       char *text;
 		       gboolean result;
 
-		       text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+		       text = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
+		       if (!text)
+			       break;
+
 		       args = process_parse_exec (text);
 
 		       result = run_file (pwin, args);
@@ -1580,9 +1553,16 @@ process_window_new (void)
 		gtk_widget_hide (get_widget (xml, "reset-profile-button"));
 	}
 	
+	pwin->time_graph = get_widget(xml, "time-graph");
+	pwin->mem_map = get_widget(xml, "mem-map");
+	g_signal_connect(pwin->time_graph, "expose_event",
+				G_CALLBACK (time_graph_expose_event), pwin);
+	g_signal_connect(pwin->mem_map, "expose_event",
+				G_CALLBACK (mem_map_expose_event), pwin);
+
 	glade_xml_signal_autoconnect (xml);
 	g_object_unref (G_OBJECT (xml));
-	
+
 	return pwin;
 }
 
